library ieee;
use ieee.std_logic_1164.all;
USE ieee.std_logic_unsigned.all;

-- Demo for ispLSI5256VA Demoboard
-- Bosshard Elektronik, CH - 8493 Saland

-- copyright notice:
-- Feel free to modify and use this file in any way. Of course we refuse
-- any liability for everything.

-- Description: This is a simple clock. The time is displayed in
-- 		24h-format hh:mm . The digits can be set by pressing
--		TA1 .. TA4




entity demo5k1 is
port (
	clk2 :			in std_logic;
	reset :			in std_logic;
	
	ta1,ta2,ta3,ta4:	in std_logic;	-- 4 Pushbuttons
	
--	sw1,sw2,sw3,sw4,sw5,sw6,sw7,sw8 :		in std_logic;

	d1a,d1b,d1c,d1d,d1e,d1f,d1g,d1dp,d1col :	buffer std_logic; --D1
	d2a,d2b,d2c,d2d,d2e,d2f,d2g,d2dp,d2col :	buffer std_logic; --D2
	d3a,d3b,d3c,d3d,d3e,d3f,d3g,d3dp,d3col :	buffer std_logic; --D3
	d4a,d4b,d4c,d4d,d4e,d4f,d4g,d4dp,d4col :	buffer std_logic  --D4

);
end;

architecture demo5k1_behav of demo5k1 is

SIGNAL clk_sec:	std_logic;			-- 1Hz Clock

SIGNAL count0:	integer range 2000000 downto 0; -- Counter for Clock-divider

SIGNAL seconds:	integer range 9 downto 0 :=0;	-- register for seconds
SIGNAL ten_sec: integer range 9 downto 0 :=0;  	-- register for tens of seconds
SIGNAL minutes: integer range 9 downto 0 :=0;	-- register for minutes
SIGNAL ten_min: integer range 9 downto 0 :=0;	-- register for tens of minutes
SIGNAL hours:	integer range 9 downto 0 :=0;	-- register for hours
SIGNAL ten_hrs: integer range 9 downto 0 :=0;	-- register for tens of hours


SIGNAL disp1 :	std_logic_vector (8 downto 0);	-- represents the bits of displ. D1
SIGNAL disp2 :	std_logic_vector (8 downto 0);	-- represents the bits of displ. D2
SIGNAL disp3 :	std_logic_vector (8 downto 0);	-- represents the bits of displ. D3
SIGNAL disp4 :	std_logic_vector (8 downto 0);	-- represents the bits of displ. D4



begin

-- clk2 is 4MHz from the onboard-osc. 
-- To get a clock with 1Hz, we have to divide by 4E6. To get a duty-cycle of 1:1 and
-- to save some registers, we complement the clock every 2E6 counts.

clock_div : process (clk2, reset)
	begin
	
	IF (reset = '0') THEN 
	count0 <= 0;
	clk_sec <= '0';
	
	ELSIF (clk2'event AND clk2 = '1') THEN
		
		IF (count0 = 2000000) THEN
			clk_sec <= NOT clk_sec;
			count0 <= 0;
		ELSE
			count0 <= count0 + 1;
		END IF;
	END IF;
END process clock_div;






display1: process (reset,clk_sec,ta1,ta2,ta3,ta4)
	begin
		IF (reset = '0') THEN
			seconds <= 0;
			ten_sec <= 0;
			minutes <= 0;
			ten_min <= 0;
			hours	<= 0;
			ten_hrs	<= 0;


		ELSIF (clk_sec'event AND clk_sec = '1') THEN

-- This is not the best solution. ta1 .. ta4 are used to count the corresponding digit
-- up one count. This is done every second, because only when we have a positive clock-
-- edge on clk_sec, we run through this part of code. When we set a digit, the seconds-
-- register is not counted up (it's ELSIFed), so we "lose" one second.
-- You should find a better way to do this and email me the solution ;-)



			IF (ta1 = '0') THEN
				IF (ten_hrs = 2) THEN ten_hrs <= 0;
				ELSE ten_hrs <= ten_hrs + 1;
				END IF;
			ELSIF (ta2 = '0') THEN
				IF (hours = 9) THEN hours <= 0;
				ELSE hours <= hours + 1;
				END IF;
			ELSIF (ta3 = '0') THEN
				IF (ten_min = 5) THEN ten_min <= 0;
				ELSE ten_min <= ten_min + 1;
				END IF;
			ELSIF (ta4 = '0') THEN
				IF (minutes = 9) THEN minutes <= 0;
				ELSE minutes <= minutes + 1;
				END IF;


			-- IF 23:59:59
			ELSIF ((ten_hrs = 2) AND (hours = 3) AND (ten_min = 5) AND (minutes = 9) AND (ten_sec = 5) AND (seconds = 9)) THEN
				seconds <= 0;
				ten_sec <= 0;
				minutes <= 0;
				ten_min <= 0;
				hours	<= 0;
				ten_hrs	<= 0;

			-- IF x9:59:59
			ELSIF ((hours = 9) AND (ten_min = 5) AND (minutes = 9) AND (ten_sec = 5) AND (seconds = 9)) THEN
				seconds <= 0;
				ten_sec <= 0;
				minutes <= 0;
				ten_min <= 0;
				hours	<= 0;
				ten_hrs	<= ten_hrs + 1;
			
			-- IF xx:59:59
			ELSIF ((ten_min = 5) AND (minutes = 9) AND (ten_sec = 5) AND (seconds = 9)) THEN
				seconds <= 0;
				ten_sec <= 0;
				minutes <= 0;
				ten_min <= 0;
				hours <= hours + 1;
			
			-- IF xx:x9:59
			ELSIF ((minutes = 9) AND (ten_sec = 5) AND (seconds = 9)) THEN
				seconds <= 0;
				ten_sec <= 0;
				minutes <= 0;
				ten_min <= ten_min + 1;
				
			-- IF xx:xx:59
			ELSIF ((ten_sec = 5) AND (seconds = 9)) THEN
				seconds <= 0;
				ten_sec <= 0;
				minutes <= minutes + 1;
				
			-- IF xx:xx:x9
			ELSIF (seconds = 9) THEN
				seconds <= 0;
				ten_sec <= ten_sec + 1;
			
			ELSE seconds <= seconds + 1;

		   	END IF;

		END IF;

end process display1;


-- This process converts the register-values to the segments of the LED-display

bcd2led: process (clk_sec)
begin
	IF (clk_sec'event and clk_sec = '0') THEN	-- falling edge of 1Hz Clock
		
		IF    (ten_hrs = 0) THEN disp1 <= "111000000";
		ELSIF (ten_hrs = 1) THEN disp1 <= "111111001";
		ELSIF (ten_hrs = 2) THEN disp1 <= "110100100";
		END IF;


		IF    (hours = 0) THEN disp2 <= "111000000";
		ELSIF (hours = 1) THEN disp2 <= "111111001";
		ELSIF (hours = 2) THEN disp2 <= "110100100";
		ELSIF (hours = 3) THEN disp2 <= "110110000";
		ELSIF (hours = 4) THEN disp2 <= "110011001";
		ELSIF (hours = 5) THEN disp2 <= "110010010";
		ELSIF (hours = 6) THEN disp2 <= "110000010";
		ELSIF (hours = 7) THEN disp2 <= "111111000";
		ELSIF (hours = 8) THEN disp2 <= "110000000";
		ELSIF (hours = 9) THEN disp2 <= "110010000";
		END IF;
		
		IF    (ten_min = 0) THEN disp3 <= "111000000";
		ELSIF (ten_min = 1) THEN disp3 <= "111111001";
		ELSIF (ten_min = 2) THEN disp3 <= "110100100";
		ELSIF (ten_min = 3) THEN disp3 <= "110110000";
		ELSIF (ten_min = 4) THEN disp3 <= "110011001";
		ELSIF (ten_min = 5) THEN disp3 <= "110010010";
		ELSIF (ten_min = 6) THEN disp3 <= "110000010";
		ELSIF (ten_min = 7) THEN disp3 <= "111111000"; -- Here you find some wasted
		ELSIF (ten_min = 8) THEN disp3 <= "110000000"; -- code. ten_min can't go higher 
		ELSIF (ten_min = 9) THEN disp3 <= "110010000"; -- than 5
		END IF;
		
		IF    (minutes = 0) THEN disp4 <= "111000000";
		ELSIF (minutes = 1) THEN disp4 <= "111111001";
		ELSIF (minutes = 2) THEN disp4 <= "110100100";
		ELSIF (minutes = 3) THEN disp4 <= "110110000";
		ELSIF (minutes = 4) THEN disp4 <= "110011001";
		ELSIF (minutes = 5) THEN disp4 <= "110010010";
		ELSIF (minutes = 6) THEN disp4 <= "110000010";
		ELSIF (minutes = 7) THEN disp4 <= "111111000";
		ELSIF (minutes = 8) THEN disp4 <= "110000000";
		ELSIF (minutes = 9) THEN disp4 <= "110010000";
		END IF;
		
	END IF;
end process bcd2led;


-- Here the digits, which have been decoded in process bcd2led are mapped to
-- the device-pins.


	d1a <= disp1(0);
	d1b <= disp1(1);
	d1c <= disp1(2);
	d1d <= disp1(3);
	d1e <= disp1(4);
	d1f <= disp1(5);
	d1g <= disp1(6);
	d1dp <= disp1(7);
	d1col <= disp1(8);

	d2a <= disp2(0);
	d2b <= disp2(1);
	d2c <= disp2(2);
	d2d <= disp2(3);
	d2e <= disp2(4);
	d2f <= disp2(5);
	d2g <= disp2(6);
	d2dp <= clk_sec;	-- d2dp blinks with 1 Hz
	d2col <= disp2(8);

	d3a <= disp3(0);
	d3b <= disp3(1);
	d3c <= disp3(2);
	d3d <= disp3(3);
	d3e <= disp3(4);
	d3f <= disp3(5);
	d3g <= disp3(6);
	d3dp <= disp3(7);
	d3col <= clk_sec;	-- d3col blinks with 1 Hz

	d4a <= disp4(0);
	d4b <= disp4(1);
	d4c <= disp4(2);
	d4d <= disp4(3);
	d4e <= disp4(4);
	d4f <= disp4(5);
	d4g <= disp4(6);
	d4dp <= disp4(7);
	d4col <= disp4(8);


end demo5k1_behav;

